MIME_TYPES = {
    'txt': 'text/plain',
    'html': 'text/html',
    'htm': 'text/html',
    'css': 'text/css',
    'md': 'text/markdown',
    'markdown': 'text/markdown',
    'csv': 'text/csv',
    
    'js': 'application/javascript',
    'mjs': 'application/javascript',

    'json': 'application/json',
    'xml': 'application/xml',

    'bin': 'application/octet-stream',

    'bz2': 'application/x-bzip2',
    'gz': 'application/gzip',
    'jar': 'application/java-archive',
    'rar': 'application/vnd.rar',
    'tar': 'application/x-tar',
    'zip': 'application/zip',
    '7z': 'application/x-7z-compressed',

    'pdf': 'application/pdf',
    'rtf': 'application/rtf',

    'docx': 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'xlsx': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'pptx': 'application/vnd.openxmlformats-officedocument.presentationml.presentation',

    'odt': 'application/vnd.oasis.opendocument.text',
    'ods': 'application/vnd.oasis.opendocument.spreadsheet',
    'odp': 'application/vnd.oasis.opendocument.presentation',

    'png': 'image/png',
    'jpg': 'image/jpeg',
    'jpeg': 'image/jpeg',
    'webp': 'image/webp',
    'svg': 'image/xvg+xml',
    'ico': 'image/x-icon',
    'bmp': 'image/bmp',
    'avif': 'image/avif',
    'tif': 'image/tiff',
    'tiff': 'image/tiff',

    'mp3': 'audio/mpeg',
    'wav': 'audio/wav',
    'weba': 'audio/webm',

    'mp4': 'video/mp4',
    'mpeg': 'video/mpeg',
    'webm': 'video/webm',

    'otf': 'font/otf',
    'ttf': 'font/ttf',
    'woff2': 'font/woff2',
    

    'aac': 'audio/aac',
    'abw': 'application/x-abiword',
    'apng': 'image/apng',
    'arc': 'application/x-freearc',
    'avi': 'video/x-msvideo',
    'azw': 'application/vnd.amazon.ebook',
    'bz': 'application/x-bzip',
    'cda': 'application/x-cdf',
    'csh': 'application/x-csh',
    'doc': 'application/msword',
    'eot': 'application/vnd.ms-fontobject',
    'ics': 'text/calendar',
    'jsonld': 'application/ld+json',
    'mid': 'audio/midi',
    'midi': 'audio/midi',
    'mpkg': 'application/vnd.apple.installer+xml',
    'oga': 'audio/ogg',
    'ogv': 'video/ogg',
    'ogx': 'application/ogg',
    'ogg': 'application/ogg',
    'opus': 'audio/ogg',
    'ppt': 'application/vnd.ms-powerpoint',
    'sh': 'application/x-sh',
    'ts': 'video/mp2t',
    'vsd': 'application/vnd.visio',
    'woff': 'font/woff',
    'xls': 'application/vnd.ms-excel',
    'xul': 'application/vnd.mozilla.xul+xml',
    '3gp': 'video/3gpp',
    '3g2': 'video/3gpp2'
}